#!/bin/bash

# include this script from the JOSEK rootpath, e.g.
# cd ~/josek
# source bin/setup.bash

function JOSEKSETUP_addpath {
  [ -d "$2" ] || return 
  if eval test -z "\"\$$1\""; then
		eval "export $1=$2"

	elif ! eval test -z "\"\${$1##*:$2:*}\"" -o -z "\"\${$1%%*:$2}\"" -o -z "\"\${$1##$2:*}\"" -o -z "\"\${$1##$2}\""; then
    eval "export $1=\$$1:$2"
  fi
}

function JOSEKSETUP_setenv {
	if eval test -z "\"\$$1\"" -o ! -x "\"\$$1\""; then
		if [ -n "$2" -a -f "$2" ]; then
			eval "export $1=$2"
		elif [ -n "$3" ]; then
			echo "Warning: could not determine a sensible setting for $1. Please set manually." 1>&2
		fi
	fi
}

if [ ! -d "$PWD/josek/code/arch" ]; then
	echo "the current directory ($PWD) does not appear to be the JOSEK root path" 1>&2
	echo "please source this script from the JOSEK root directory (contains the josek and bin subdirs)" 1>&2
else

	export JOSEK_BASE="$PWD"
	export JOSEKRULES="$JOSEK_BASE/josek/code/arch/"
	JOSEKSETUP_addpath PATH "${JOSEK_BASE}/bin"

	# JDK setup
	# JDK tools.jar is needed since jino uses the javac Sun API, which is not
	# part of the public API	
	JOSEKSETUP_setenv JAVACC "`which javacc`" WARN

	if [ -n $JDK ] && [ -f ${JDK}/lib/tools.jar ]; then
		echo "Using preconfigured JDK at $JDK"

		######         Mac OS X 10.6
	elif [ `uname` = 'Darwin' ]; then
		if [ -d "/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0" ]; then
			export JDK=/System/Library/Frameworks/JavaVM.framework/Versions/1.6.0
			export JDKTOOLS=$JDK/Classes/classes.jar
			JOSEKSETUP_setenv JAVA "${JDK}/Commands/java" WARN
			JOSEKSETUP_setenv JAVAC "${JDK}/Commands/javac" WARN
		fi

		if [ -n "$JDK" -o ! -f ${JDK}/Classes/classes.jar ]; then
			echo "cannot find a valid JDK" 1>&2
			echo "please correct the setting of the JDK environment variable" 1>&2
			echo "the JDK path should contain the Classes/classes.jar file" 1>&2
			unset JDK
		fi

	else
		for jdkpath in \
			/local/java \
			/usr/lib/jvm/java-6-sun \
			/usr/lib/jvm/java-6-openjdk \
			; do

			if [ -f "$jdkpath/lib/tools.jar" ]; then
				export JDK="$jdkpath"
				break
			fi
		done

		if [ -z "$JDK" -o ! -f "${JDK}/lib/tools.jar" ]; then
			echo "cannot find a valid JDK $JDK" 1>&2
			echo "please correct the setting of the JDK environment variable" 1>&2
			echo "the JDK path should contain the lib/tools.jar file" 1>&2
			unset JDK
		else
			JOSEKSETUP_setenv JAVA "${JDK}/bin/java"
			JOSEKSETUP_setenv JAVAC "${JDK}/bin/javac"
			export JDKTOOLS=${JDK}/lib/tools.jar
		fi
	fi
fi

unset -f JOSEKSETUP_addpath
unset -f JOSEKSETUP_setenv

