/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;
import java.util.Set;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;

public class ArrayOptimizer {
	private String name;
	private String xType;
	private String yType;
	private TreeMap values;
	private boolean inRAM;
	private boolean finalized;
		
	public ArrayOptimizer(String name, String xType, String yType, boolean inRAM) {
		values = new TreeMap();
		this.name = name;
		this.xType = xType;
		this.yType = yType;
		this.inRAM = inRAM;
		finalized = false;
	}

	public void addValue(int a, int b) {
		if (finalized) throw new RuntimeException("Cannot add values to finalized array.");
		values.put(new Integer(a), new Integer(b));
	}
	
	public void addValue(int a, String b) {
		if (finalized) throw new RuntimeException("Cannot add values to finalized array.");
		values.put(new Integer(a), b);
	}

	public void finalize() {
		finalized = true;
	}

	public void generateDefine() {
		SSingleton.clear();

		if (!inRAM) {
			/* This is a function, include prototype to set attribute */
			SSingleton.add(yType);
			SSingleton.add(" ");
			SSingleton.add(name);
			SSingleton.add("Values(");
			SSingleton.add(xType);
			SSingleton.add(" x) __attribute__((const));");
			SSingleton.newLine();
		}

		SSingleton.add("#define ");
		SSingleton.add(name);
		SSingleton.add("(x)		(");
		SSingleton.add(name);
		SSingleton.add("Values");
	
		if (!inRAM) {
			SSingleton.add("(x)");
		} else if(values.size()>1) {
			SSingleton.add("[x]");
		}
		SSingleton.add(")");
	}

	public boolean generateArray() {
		if (!inRAM) return false;
		SSingleton.clear();
		SSingleton.add("static ");
		SSingleton.add(yType);
		SSingleton.add(" ");
		SSingleton.add(name);
		SSingleton.add("Values");

		if(values.size()>1)
			SSingleton.add("[] = {");
		else
			SSingleton.add("=");

		Set s = values.entrySet();
		int j = 0;
		for (Iterator i = s.iterator(); i.hasNext(); ) {
			Entry e = (Entry)i.next();
			int key = ((Integer)(e.getKey())).intValue();
			Object val = e.getValue();

			if (j++ != key) throw new RuntimeException("TODO: Using arbitary x/y-mappings using array currently not implemented.");
			SSingleton.add(val.toString());
			if (i.hasNext()) SSingleton.add(", ");
		}

		if(values.size()>1)
			SSingleton.add("}");
		SSingleton.add(";");
		return true;
	}
	
	public boolean generateFunction() {
		if (inRAM) return false;
		SSingleton.clear();
		SSingleton.add(yType);
		SSingleton.add(" ");
		SSingleton.add(name);
		SSingleton.add("Values(");
		SSingleton.add(xType);
		SSingleton.add(" x) {");
		SSingleton.incIndent();
		SSingleton.newLine();

		SSingleton.add("switch (x) {");
		SSingleton.incIndent();
		SSingleton.newLine();
	
		Set s = values.entrySet();
		for (Iterator i = s.iterator(); i.hasNext(); ) {
			Entry e = (Entry)i.next();
			int key = ((Integer)(e.getKey())).intValue();
			Object val = e.getValue();
			String putString;
			if (val instanceof Integer) {
				putString = ((Integer)(val)).toString();
			} else if (val instanceof String) {
				putString = (String)val;
			} else {
				throw new RuntimeException("Internal error.");
			}
			SSingleton.add("case ");
			SSingleton.add(key);
			SSingleton.add(": return ");
			SSingleton.add(putString);
			SSingleton.add(";");
			if (i.hasNext()) SSingleton.newLine();
		}
		
		SSingleton.decIndent();
		SSingleton.newLine();
		SSingleton.add("}");
		SSingleton.newLine();
		
		SSingleton.add("abort();	/* Array index out of bounds */");
		SSingleton.newLine();
		
		SSingleton.add("return 0;");
		SSingleton.decIndent();
		SSingleton.newLine();
		
		SSingleton.add("}");
		SSingleton.newLine();
		return true;
	}
}

