/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;
import java.io.*;

abstract class CodeHook_Standard implements CodeHook {
	private String condition;
	private BufferedWriter file;
	protected HighlevelConfiguration conf;
	protected CommandLineOptions cmdLine;
	
	protected void writeLine(String line) {
		try {
			file.write(line);
			file.write('\n');
		} catch (IOException e) {
			throw new RuntimeException("Error writing line: " + e);
		}
	}
	
	protected void writeSingStr() {
		try {
			file.write(new String(SSingleton.get()));
			file.write('\n');
		} catch (IOException e) {
			throw new RuntimeException("Error writing line: " + e);
		}
	}
	
	public CodeHook init(String condition, HighlevelConfiguration conf, CommandLineOptions cmdLine) {
		this.condition = condition;
		this.conf = conf;
		this.cmdLine = cmdLine;
		return this;
	}

	public void changedFile(BufferedWriter file) {
		this.file = file;
	}

	abstract public void fireImmediately();
	
	public boolean fire(String condition) {
		if (condition.equals(this.condition)) {
			if (cmdLine.isDefined("codecomments")) writeLine("/**** Code included from hook " + condition + " ****/");
			fireImmediately();
			if (cmdLine.isDefined("codecomments")) writeLine("/**** Done with code included from hook " + condition + " ****/");
			return true;
		}
		return false;
	}
}

