/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;

import java.io.*;
import java.util.Enumeration;

public class CodeHook_osc_counters extends CodeHook_Standard {
	public void fireImmediately() {
		if (conf.getCounterCount() == 0) return;
	
		SSingleton.clear();
		if (conf.getCounterCount() == 1) {
			SSingleton.addNL("static TickType CntrCurrentValue=0;");
			SSingleton.addNL("static TickType CntrTickValue=0;");
			SSingleton.addNL("#define CntrCurrentValue(cid) CntrCurrentValue");
			SSingleton.addNL("#define CntrTickValue(cid) CntrTickValue");
		} else {
			SSingleton.add("static TickType CntrCurrentValue[] = { ");
			for (int i = 0; i < conf.getCounterCount(); i++) {
				SSingleton.add("0");
				if (i != conf.getCounterCount() - 1) SSingleton.add(", ");
			}
			SSingleton.add(" };");
			writeSingStr();

			SSingleton.clear();
			SSingleton.add("static TickType CntrTickValue[] = { ");
			for (int i = 0; i < conf.getCounterCount(); i++) {
				SSingleton.add("0");
				if (i != conf.getCounterCount() - 1) SSingleton.add(", ");
			}
			SSingleton.addNL(" };");
			SSingleton.addNL("#define CntrCurrentValue(cid) CntrCurrentValue[cid]");
			SSingleton.addNL("#define CntrTickValue(cid) CntrTickValue[cid]");
		}
		writeSingStr();
		
		if (conf.getAlarmCount() == 0) return;

		SSingleton.clear();
		if (conf.getAlarmCount() == 1) {
			HighlevelConfiguration_Alarm alrm = conf.getAlarms().nextElement();
			SSingleton.add("static TickType AlarmTicks = ");
			SSingleton.add(alrm.getAlarmTime());
			SSingleton.addNL(";");
			SSingleton.add("static TickType AlarmCycle = ");
			SSingleton.add(alrm.getCycleTime());
			SSingleton.addNL(";");
			SSingleton.addNL("#define AlarmTicks(aid) AlarmTicks");
			SSingleton.addNL("#define AlarmCycle(aid) AlarmCycle");
		} else {
			int[] at = new int[conf.getMaxAlarmID()+1];
			int[] ac = new int[conf.getMaxAlarmID()+1];

			for(Enumeration<HighlevelConfiguration_Alarm> e = conf.getAlarms();
					e.hasMoreElements(); ) {
				HighlevelConfiguration_Alarm alrm = e.nextElement();
				System.err.println("ID: " + alrm.getInternalID());
				at[alrm.getInternalID()] = alrm.getAlarmTime();
				ac[alrm.getInternalID()] = alrm.getCycleTime();
			}

			SSingleton.add("static TickType AlarmTicks[] = { ");
			for(int i=0; i<at.length; i++) {
				SSingleton.add(at[i]);
				if (i < at.length-1) SSingleton.add(", ");
			}
			SSingleton.add(" };");
			writeSingStr();

			SSingleton.clear();
			SSingleton.add("static TickType AlarmCycle[] = { ");
			for(int i=0; i<ac.length; i++) {
				SSingleton.add(ac[i]);
				if (i < ac.length-1) SSingleton.add(", ");
			}
			SSingleton.addNL(" };");
			SSingleton.addNL("#define AlarmTicks(aid) AlarmTicks[aid]");
			SSingleton.addNL("#define AlarmCycle(aid) AlarmCycle[aid]");
		}
		writeSingStr();
	}
}
