/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;

import java.io.*;
import java.util.Iterator;

/**
 * This CodeHook generates the readyqueues if needed.
 */
public class CodeHook_osc_readyqueues extends CodeHook_Standard {
	public void fireImmediately() {
		if (!cmdLine.isDefined("have_readyqueues")) return;
	
		SSingleton.clear();

		SSingleton.add("static struct bbqueue bbqstate[");
		SSingleton.add(conf.getNumPriorities());
		SSingleton.addNL("] = {");
		int pos=0;
		StringBuffer rdyq = new StringBuffer();

		for(Iterator<Priority> pit=conf.getPriorityIterator(); pit.hasNext(); ) {
			Priority p = pit.next();
			
			// initial content of the readyqueue
			int usedSlots=0;
			/*
			for(int i=0; i<p.numChildren(); i++) {
				HighlevelConfiguration_Task tobj = p.getChild(i);
				if(tobj.getAutoactivation()) {
					rdyq.append(tobj.getName());
					rdyq.append(", ");
					usedSlots++;
				}
			}
			*/
			for(int i=usedSlots ; i<p.getMaxActivations(); i++ ) {
				rdyq.append("0, ");
			}

			// initial state of the readyqueue
			SSingleton.add("\t{ ");
			SSingleton.add(pos); // head index
			SSingleton.add(", ");
			SSingleton.add(pos/* + (usedSlots % p.getMaxActivations())*/ ); // tail index (next free slot for insertion)
			SSingleton.add(", ");
			SSingleton.add(0 /*usedSlots*/); // fill
			SSingleton.add(", ");
			SSingleton.add(pos); // start index
			SSingleton.add(", ");
			pos += p.getMaxActivations();
			SSingleton.add(pos-1); // end index
			SSingleton.addNL(" },");
		}
		SSingleton.addNL("};");

		SSingleton.add("static TaskType readyqueue[");
		SSingleton.add(pos);
		SSingleton.add("] = { ");
		SSingleton.add(rdyq.toString());
		SSingleton.addNL(" };");

		writeSingStr();

	}
}
