/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;

import java.io.*;

import java.util.Iterator;
import java.util.Enumeration;

public class CodeHook_osh_symbols extends CodeHook_Standard {
	public void fireImmediately() {
		StringBuffer s;
		
		/* Task Identifiers */
		s = new StringBuffer("typedef enum TASKS { ");
		s.append("INVALID_TASK=");
		s.append(conf.getMaxTaskID()+1);

		for (Enumeration<HighlevelConfiguration_Task> e=conf.getTasks();
				e.hasMoreElements(); ) {
			HighlevelConfiguration_Task tobj = e.nextElement();
			s.append(", ");
			s.append(tobj.getName());
			s.append("=");
			s.append(tobj.getInternalID());
		}
		s.append(" } TaskType;\n");
		s.append(conf.getTaskPriorityRevMapMacro());
		writeLine(new String(s));
	
		/* Resource Identifiers
		 *
		 * Resources are assigned their ceiling priority as identifier. This
		 * automatically folds multiple resources with identical ceiling
		 * priorities.
		 */
		if (cmdLine.isDefined("have_resources") && conf.getResourceCount() > 0) {
			s = new StringBuffer("enum RESOURCES { ");

			for (Enumeration<HighlevelConfiguration_Resource> e=conf.getResources();
					e.hasMoreElements(); ) {
				HighlevelConfiguration_Resource robj = e.nextElement();
				s.append(robj.getName());
				s.append("=");
				s.append(robj.getCeilingPriority());
				if (e.hasMoreElements()) s.append(", ");
			}
			s.append(" };");
			writeLine(new String(s));
		}

		/* Counter Identifiers */
		if(conf.getCounterCount()>0) {
			SSingleton.clear();
			s = new StringBuffer("typedef enum { ");
			for(Iterator<HighlevelConfiguration_Counter> it=conf.getCounterIterator(); it.hasNext(); ) {
				HighlevelConfiguration_Counter ctr = it.next();
				s.append(ctr.getName());
				s.append(" = ");
				s.append(ctr.getInternalID());
				if (it.hasNext()) s.append(", ");
			}
			s.append(" } CounterType;");
			writeLine(new String(s));

			SSingleton.add("typedef enum { ");
			for(Enumeration<HighlevelConfiguration_Alarm> e = conf.getAlarms(); e.hasMoreElements() ; ) {
				HighlevelConfiguration_Alarm alrm = e.nextElement();
				SSingleton.add(alrm.getName());
				SSingleton.add(" = ");
				SSingleton.add(alrm.getInternalID());
				if(e.hasMoreElements()) SSingleton.add(", ");
			}
			SSingleton.addNL(" } AlarmType;");
			SSingleton.add(conf.getAlarmCounterRevMapMacro());
			writeSingStr();
		}

		if (conf.getISRCount() > 0 && cmdLine.getArchitecture().hasISRLevels()) {
			SSingleton.clear();
			SSingleton.add("enum ISRNAMES { ");
			for (int i = 0; i < conf.getISRCount(); i++) {
				SSingleton.add(conf.ISRLevels.getLevel(i));
				if (i != conf.getISRCount() - 1) SSingleton.add(", ");
			}
			SSingleton.add(" };");
			writeSingStr();

		}
		
		writeLine("");
		
		for(Enumeration<HighlevelConfiguration_Task> e=conf.getTasks();
				e.hasMoreElements(); ) {
			if(cmdLine.isDefined("have_ecc")) {
				throw new RuntimeException("ECC not supported");
				/*
				s = new StringBuffer("#define STACKSIZE_TASK_");
				s.append(conf.getTask(i).getName());
				s.append(" ");
				s.append(conf.getTask(i).getStackSize());
				writeLine(new String(s));
				*/
			}

			s = new StringBuffer("extern void JOSEK_TASK_");
			s.append(e.nextElement().getName());
			s.append("(void);");
			writeLine(new String(s));
		}
		writeLine("");

		if (conf.getEventCount() > 0) {
			for (int i = 0; i < conf.getEventCount(); i++) {
				s = new StringBuffer("#define ");
				s.append(conf.getEvent(i).getName());
				s.append(" ");
				s.append(conf.getEvent(i).getEventMask());
				writeLine(new String(s));
			}
			writeLine("");
		}
		
		if (conf.getIOISRCount() > 0) {
			s = new StringBuffer("#define NUMBER_IO_ISR ");
			s.append(Integer.toString(conf.getIOISRCount()));
			writeLine(new String(s));
		}
		
		writeLine("#define NUMTASKS " + conf.getTaskCount());
		writeLine("#define NUMBER_RESOURCES " + conf.getResourceCount());
		writeLine("#define NUMBER_PRIORITIES " + conf.getNumPriorities());
		writeLine("#define NUMBER_COUNTERS " + conf.getCounterCount());
		writeLine("#define MAX_TASK_ACTIVE INVALID_TASK");
		writeLine("#define PRIO_IDLE_TASK NUMBER_PRIORITIES");
		
		writeLine("");
	}
}
