/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;

import josek.idmap.IDMapObject;
import josek.idmap.IDMapBaseObject;
import java.util.Vector;

public class HighlevelConfiguration_Counter extends IDMapBaseObject<HighlevelConfiguration_Alarm> {
	private int cntrMaxValue;
	private int minCycle;
	private int ticksPerBase;
	private long timeInNanoSeconds;
	private String hwbase;

	private static int ticktype_size=8;
	private static int counter_id=0;

	HighlevelConfiguration_Counter(String name, int cntrMaxValue, int minCycle,
			int ticksPerBase, long timeInNanoSeconds, String hwbase) {
		super(counter_id++, name);
		this.cntrMaxValue = cntrMaxValue;
		this.minCycle = minCycle;
		this.ticksPerBase = ticksPerBase;
		this.timeInNanoSeconds = timeInNanoSeconds;
		this.hwbase=hwbase;

		if(cntrMaxValue<256 && ticksPerBase<255) return;
		if(ticktype_size< 16) ticktype_size=16;
		if(cntrMaxValue<65536 && ticksPerBase<65536) return;
		if(ticktype_size<32) ticktype_size=32;
	}

	public static String getTickType() {
		return "uint" + ticktype_size + "_t";
	}

	public String getHWBase() {
		if(hwbase==null)
			throw new RuntimeException("TIMER_BASE not specified for counter " + name);
		return hwbase;
	}

	public int getMaxValue() {
		return cntrMaxValue;
	}

	public int getTicksPerBase() {
		return ticksPerBase;
	}

	public long getTimeInNanoSeconds() {
		return timeInNanoSeconds;
	}

	public void setMultiplicator(int m) {
		ticksPerBase *= m;
	}

	public String toString() {
		return "Counter '" + name + "': Max=" + cntrMaxValue + ", MinPerMaj=" + ticksPerBase + ", Min=" + timeInNanoSeconds + "ns";
	}
};
