/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;
import java.util.Vector;
import java.lang.Comparable;

import josek.idmap.IDMapChildObject;

public class HighlevelConfiguration_Task extends IDMapChildObject<Priority> {
	private static int schedMode=0;

	private final boolean autoActivation;
	private final int priority;
	private final int stacksize;
	private final boolean preempt_sched;
	private final int mact;
	private final Vector<String> usedResources;
	private final Vector<String> usedEvents;
	
	HighlevelConfiguration_Task(String name, boolean autoActivation, int priority,
			int stacksize, int mact, boolean preemptive, Vector<String> usedResources,
			Vector<String> usedEvents) {
		super(-1, name);
		this.autoActivation = autoActivation;
		this.priority = priority;
		this.usedResources = usedResources;
		this.usedEvents = usedEvents;
		this.stacksize=stacksize;
		this.mact = mact;
		this.preempt_sched=preemptive;
		schedMode |= (preemptive ? 1 : 2);
	}

	public boolean preemptive() { return this.preempt_sched; }

	public int getMaxActivation() { return mact; }

	public boolean getAutoactivation() { return autoActivation; }

	public int getStackSize() { return stacksize; }

	public int getUsedResourceCount() {
		return usedResources.size();
	}

	public String getUsedResource(int i) {
		return usedResources.get(i);
	}

	public int getUsedEventCount() {
		return usedEvents.size();
	}

	public String getUsedEvent(int i) {
		return usedEvents.get(i);
	}

	public String toString() {
		String s = "Task '" + super.toString() + "':\n    Autoactivation: ";
		if (autoActivation) s += "Yes"; else s += "No";
		s += "\n    Priority      : " + priority;
		s += "\n    # of resources: " + getUsedResourceCount();
		s += "\n    # of events   : " + getUsedEventCount();
		return s;
	}

	static int getSchedMode() {
		return schedMode;
	}
};
