/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;

public class ISRLevels {
	private String[] levelNames;
	private HighlevelConfiguration conf;

	public ISRLevels(HighlevelConfiguration conf) {
		this.conf = conf;
		if (conf.getISRCount() == 0) return;

		levelNames = new String[conf.getISRCount()];
		for (int i = 0; i < conf.getISRCount(); i++) {
			registerISR(conf.getISR(i));
		}
	}

	private void registerISR(HighlevelConfiguration_ISR isr) {
		String name = isr.getName();
		int lvl = isr.getLevel();
		if (lvl == -1) {
			int j;
			for (int i = 0; i < conf.getISRCount(); i++) if (levelNames[i] == null) {
				lvl = i;
				break;
			}
			System.err.println("Warning: Autoassigned ISR " + name + " IRQ level " + lvl + ", please specify explicitly.");
		}

		if (lvl >= conf.getISRCount()) {
			throw new RuntimeException("IRQ level of ISR " + name + " is not within bounds (must be from 0 to " + (conf.getISRCount() - 1) + ")");
		}
		if (levelNames[lvl] != null) {
			throw new RuntimeException("IRQ level of ISR " + name + " is already taken by ISR " + levelNames[lvl]);
		}
		levelNames[lvl] = name;
	}

	public void getArray(ArrayOptimizer a) {
		for (int i = 0; i < conf.getISRCount(); i++) {
			a.addValue(i, "JOSEK_ISR_" + levelNames[i]);
		}
	}

	public String getLevel(int i) {
		return levelNames[i];
	}
}
