/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek;

import josek.idmap.IDMapBaseObject;
import josek.idmap.IDMapObject;

import java.util.Vector;
import java.util.Collections;

/**
 * This class represents a priority level.
 *
 * Contrary to OSEK priority levels, internal JOSEK priority levels
 * have lower values for higher priority. This is due to the design
 * of the scheduler, and has to be taken into account when calculating
 * ceiling priorities etc.
 */
class Priority extends IDMapBaseObject<HighlevelConfiguration_Task> {
	private int maxActivations; /* Max. number of ready tasks on this priority level */
	private boolean isCeilingPrio;

	Priority(int p, int initact, String name) {
		super(p, name);
		maxActivations = initact;
		isCeilingPrio=false;
	}

	void setIsCeilingPrio() {
		if(!isCeilingPrio) {
			isCeilingPrio=true;
			// maxActivations++; /* We do not insert tasks holding resources in higher prio readyqueues but maintain a separate stack */
		}
	}

	void incActivations(int act) {
		maxActivations += act;
	}

	public int getMaxActivations() { return maxActivations; }

	/**
	 * We override the compareTo method for priorities to
	 * generate small internal priorities for high external
	 * priorities.
	 *
	 * This is because task ids, that are derived from priority
	 * values, are to be used for indexing arrays and should
	 * start at zero, however, the least priority is that of
	 * the idle task which is not present in these arrays and
	 * thus the array would unneccessarily grow if we kept the
	 * original priority order.
	 */
	public int compareTo(IDMapObject imo) {
		if(externalID != imo.getExternalID())
			return imo.getExternalID() - externalID;
		return name.compareTo(imo.getName());
	}
}

