/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek.arch;

import josek.CommandLineOptions;
import josek.HighlevelConfiguration;

public abstract class Architecture {
	protected CommandLineOptions cmdLine;
	protected HighlevelConfiguration conf;

	public String getCFlags() {
		return "-Os -Wall -Werror -ansi -pedantic -combine -fwhole-program -fno-keep-static-consts -fomit-frame-pointer";
	}

	public final void setCommandLine(CommandLineOptions co) {
		cmdLine = co;
	}

	public final void setHighlevelConfig(HighlevelConfiguration hc) {
		conf = hc;
	}

	public void setArchOptions() {
		if(hasLinux()) cmdLine.defineSymbol("have_linux");
		if(hasStdlib()) cmdLine.defineSymbol("have_stdlib");
	}

	public abstract int machineWordSize();

	// Base layer support
	public boolean hasLinux() { return true ; }
	public boolean hasStdlib() { return true; }

	// Scheduler support
	public boolean supportsPreemption() { return true; }
	public boolean supportsECC() { return true; }

	// Hardware timer support
	public abstract void init_timer(StringBuffer s);
	public abstract void emit_timerhandlers(StringBuffer s);
	
	// ISR support
	public abstract boolean hasISRLevels();
}

