/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/

package josek.arch;

import josek.HighlevelConfiguration_Counter;

import java.util.Iterator;

public abstract class Linux extends Architecture {
	// Hardware timer support
	protected long maxNanoSeconds() {
		return 100000000000000000L;
	}
	protected long minNanoSeconds() {
		return 10000L; // 10 ms
	}

	private void checkTimerInterval(long ns) {
		if(ns<minNanoSeconds() || ns>maxNanoSeconds())
			throw new RuntimeException("Timer interval " + ns
					+ "ns not in the supported interval [" + minNanoSeconds()/1000
					+ "ms; " + maxNanoSeconds()/1000000000 + "s]");
	}
	
	public void init_timer(StringBuffer s) {
		if(conf.getCounterCount()>1)
			throw new RuntimeException("Only one Hardware Counter supported on Linux for now");

		HighlevelConfiguration_Counter ctr = conf.getCounterIterator().next();
		long ns_interval = ctr.getTimeInNanoSeconds();
		checkTimerInterval(ns_interval);

		int seconds  = (int)(ns_interval/1000000000L);
		long restNS  = ns_interval%1000000000L;
		int uSeconds = (int) (restNS / 1000L);

		s.append("\tstruct itimerval tim;\n");
		s.append("\ttim.it_interval.tv_sec = " + seconds +";\n");
		s.append("\ttim.it_value.tv_sec = " + seconds + ";\n");
		s.append("\ttim.it_interval.tv_usec = " + uSeconds +";\n");
		s.append("\ttim.it_value.tv_usec = " + uSeconds + ";\n");
		s.append("\tif (setitimer(ITIMER_REAL, &tim, NULL)) perror(\"setitimer\");\n");

		cmdLine.verbln("Linux Timer configured with deviation " + (ns_interval % minNanoSeconds()) + "ns");
	}

	public void emit_timerhandlers(StringBuffer s) {
		if(conf.getCounterCount()>1)
			throw new RuntimeException("Only one Hardware Counter supported on Linux for now");

		HighlevelConfiguration_Counter ctr = conf.getCounterIterator().next();
		s.append("void alarm_handler (int s) {\n");
		s.append("\tadvanceCounter(");
		s.append(ctr.getName());
		s.append(");\n");
		s.append("}");
	}

	public boolean hasISRLevels() { return true; }

	public String getCFlags() {
		StringBuffer cflags=new StringBuffer(super.getCFlags());
		cflags.append(" -D_XOPEN_SOURCE=500");
		return cflags.toString();
	}

}
