/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek.arch.atmega;

import josek.HighlevelConfiguration;
import josek.HighlevelConfiguration_Counter;

class atmega644 extends ATMega {
	private int timsk0,timsk1,timsk2;
	
	atmega644() {
		counters_used=new boolean[3];
		timer_prescalers = new int[][] { pscalers, pscalers, epscalers };
		timer_max = new int[] { 255, 65535, 255 };
	}

	protected void set_timsk(HighlevelConfiguration_Counter ctr, int ti) {
		if(ti==2)
			timsk2 |= 2;
		else if(ti==1)
			timsk1 |= 2;
		else if(ti==0)
			timsk0 |=2;
		else throw new
			RuntimeException("Interrupt Mask position unknown for timer " + ctr.getHWBase());
	}

	protected void set_tccr(HighlevelConfiguration_Counter ctr, int ti, int prescalerIndex) {
		if(timer_max[ti] == 255) { // 8-bit timer
			tccr[ti] = 2;
			tccrb[ti] = (prescalerIndex+1);
		} else { // 16-bit timer
			tccr[ti] = 0x00;
			tccrb[ti] = 0x08 | (prescalerIndex+1);
		}
	}

	public void addhandler(StringBuffer s, HighlevelConfiguration_Counter ctr) {
		int ti=tind(ctr.getHWBase());
		s.append("ISR(TIMER");
		s.append(ti);
		s.append("_COMPA_vect) {\n");
		s.append("\tadvanceCounter(");
		s.append(ctr.getName());
		s.append(");\n");
		s.append("}");
	}

	public void write_init(StringBuffer s) {
		for(int i=0; i<timer_max.length; i++) {
			if(! counters_used[i]) continue;

			s.append("\tTCCR" + i + "B = 0x" + Integer.toHexString(tccrb[i]) + ";\n");
			s.append("\tTCCR" + i + "A = 0x" + Integer.toHexString(tccr[i]) + ";\n");
			s.append("\tOCR" + i + "A = " + ocr[i] + ";\n");
			s.append("\tTCNT" + i + " = 0;\n");
		}
		if(timsk0 != 0)
			s.append("\tTIMSK0 |= 0x" + Integer.toHexString(timsk0) + ";\n");
		if(timsk1 != 0)
			s.append("\tTIMSK1 |= 0x" + Integer.toHexString(timsk1) + ";\n");
		if(timsk2 != 0)
			s.append("\tTIMSK2 |= 0x" + Integer.toHexString(timsk2) + ";\n");
	}
}

