/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek.arch;

import josek.HighlevelConfiguration;
import josek.HighlevelConfiguration_Counter;
import josek.arch.atmega.ATMega;

import java.util.Iterator;

public final class avr extends Architecture {
	private int __freq=0;
	public int freq() {
		if(__freq==0) {
			String freqs = conf.getCodeHookVariable("mcufreq");
			if(freqs==null)
				throw new RuntimeException("Define MCU frequency FREQ to use hardware counter");
			__freq = Integer.parseInt(freqs);
		}
		return __freq;
	}

	public String getCFlags() {
		StringBuffer cflags=new StringBuffer(super.getCFlags());
		cflags.append(" -mmcu=");
		cflags.append(conf.getCodeHookVariable("mcu"));
		cflags.append(" -ffreestanding -fshort-enums");
		return cflags.toString();
	}

	public String toString() { return "avr"; }

	// Base layer support
	public boolean hasLinux() { return false; }
	public boolean hasStdlib() { return false; }
	public int machineWordSize() { return 8; }

	// Scheduler support
	public boolean supportsPreemption() { return true; }
	public boolean supportsECC() { return false; }

	// Hardware timer support
	private static final int[] prescalers = { 1, 8, 64, 256, 1024 };

	// Timer0 on the ATMega128, Timer 2 on the ATMega32
	private static final int[] extprescalers = { 1, 8, 32, 64, 128, 256, 1024 };

	public void init_timer(StringBuffer s) {
		ATMega mcu = ATMega.MCUDesc(conf,cmdLine, this);

		for(Iterator<HighlevelConfiguration_Counter> it=conf.getCounterIterator();
				it.hasNext(); ) {
			mcu.init(it.next());
		}
		mcu.write_init(s);
	}

	public void emit_timerhandlers(StringBuffer s) {
		ATMega mcu = ATMega.MCUDesc(conf,cmdLine, this);

		for(Iterator<HighlevelConfiguration_Counter> it=conf.getCounterIterator();
				it.hasNext(); ) {
			mcu.addhandler(s, it.next());
		}
	}
	public boolean hasISRLevels() { return false; }
}
