/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
//? false
/* Ceiling priorities are kept on a stack. According to the OSEK Spec,
 * Get/ReleaseResource do not perform any error checking in Standard
 * mode. Resources must be aquired and released with proper nesting.
 *
 * Data Structures:
 * 	JOSEK_used_resources_stack: stack of ceiling priorities. The stack
 * 	  has one base entry that will never be popped, a virtual priority
 * 	  one less than the idle priority. This will allow checking the
 * 	  ceiling priority without checking the stack index first.
 *	JOSEK_used_resources_index: stack index
 *
 * Since we do not need to perform error checking, we only really
 * aquire a resource when its ceiling priority is higher or equal
 * than that on the top of the ceiling stack. If it isn't, acquiring
 * the resource does not incur side effects, since it must be
 * released before the resource with the higher ceiling priority.
 *
 * Resource identifiers are equal to their ceiling priorities.
 */
//?
uint8_t GetResource(ResourceType res) {
	if(JOSEK_used_resources_stack[JOSEK_used_resources_index] >= res) {
		JOSEK_used_resources_stack[++JOSEK_used_resources_index] = res;
	}
	return E_OK;
}

uint8_t ReleaseResource(ResourceType res) {
	if(JOSEK_used_resources_stack[JOSEK_used_resources_index] == res) {
		JOSEK_used_resources_index--;
		schedule(0);
	}

	return E_OK;
}

