/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
/* Generated By:JavaCC: Do not edit this line. ConditionParser.java */
package josek.conditionparser;

import java.io.StringReader;
import java.io.Reader;

import josek.CommandLineOptions;

public class ConditionParser implements ConditionParserConstants {
        interface Condition {
                public boolean eval();
        }
        interface NodeContainer {
                public void setCond(Condition c);
                public Condition getCond();
        }

        class TrueCondition implements Condition {
                public boolean eval() { return true; }
        }
        class FalseCondition implements Condition {
                public boolean eval() { return false; }
        }
        class SingleChild implements Condition {
                Condition child;
                public boolean eval() { return child.eval(); }
        }
        class NegCond extends SingleChild {
                public boolean eval() { return !child.eval(); }
        }
        class SingleSetter implements NodeContainer {
                SingleChild p;
                SingleSetter(SingleChild parent) { p=parent; }
                public void setCond(Condition c) { p.child=c; }
                public Condition getCond() { return p.child; }
        }

        abstract class DoubleChild implements Condition {
                Condition left, right;
                public abstract boolean eval();
        }
        class AndCondition extends DoubleChild {
                public boolean eval() { return left.eval() && right.eval(); }
        }
        class OrCondition extends DoubleChild {
                public boolean eval() { return left.eval() || right.eval(); }
        }
        class LeftSetter implements NodeContainer {
                DoubleChild dc;
                LeftSetter(DoubleChild d) { dc=d; }
                public void setCond(Condition c) { dc.left=c; }
                public Condition getCond() { return dc.left; }
        }
        class RightSetter implements NodeContainer {
                DoubleChild dc;
                RightSetter(DoubleChild d) { dc=d; }
                public void setCond(Condition c) { dc.right=c; }
                public Condition getCond() { return dc.right; }
        }

        SingleChild rootcdt;
        SingleSetter rootsetter;
        CommandLineOptions cmdLine;

        public ConditionParser(String s, CommandLineOptions clo) {
                this((Reader) new StringReader(s));
                rootcdt = new SingleChild();
                rootsetter = new SingleSetter(rootcdt);
                cmdLine=clo;
        }

        public static void main(String[] args) {
                if(eval(args[0],null))
                        System.out.println("true");
                else
                        System.out.println("false");
        }
        public static boolean eval(String cdt, CommandLineOptions clo) {
                try {
                        ConditionParser parser = new ConditionParser(cdt,clo);
                        parser.expression(parser.rootsetter);
                        return parser.rootcdt.eval();
                } catch(ParseException e) {
                        e.printStackTrace();
                        throw new RuntimeException("Illegal condition: " + cdt);
                }
        }

  final public void expression(NodeContainer ctr) throws ParseException {
        DoubleChild dc;
        RightSetter rs;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case LPAREN:
    case IDENTIFIER:
      symbol(ctr);
      break;
    case NEG:
      negsymbol(ctr);
      break;
    default:
      jj_la1[0] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    label_1:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case AND:
      case OR:
        ;
        break;
      default:
        jj_la1[1] = jj_gen;
        break label_1;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case AND:
        jj_consume_token(AND);
                                        dc = new AndCondition();
        break;
      case OR:
        jj_consume_token(OR);
                                        dc = new OrCondition();
        break;
      default:
        jj_la1[2] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
                                dc.left = ctr.getCond();
                                ctr.setCond(dc);
                                rs = new RightSetter(dc);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case LPAREN:
      case IDENTIFIER:
        symbol(rs);
        break;
      case NEG:
        negsymbol(rs);
        break;
      default:
        jj_la1[3] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void symbol(NodeContainer ctr) throws ParseException {
        Token symName;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case IDENTIFIER:
      symName = jj_consume_token(IDENTIFIER);
                        if(symName.image.equals("false"))
                                ctr.setCond(new FalseCondition());
                        else if(symName.image.equals("true"))
                                ctr.setCond(new TrueCondition());
                        else if(cmdLine.isDefined(symName.image))
                                ctr.setCond(new TrueCondition());
                        else
                                ctr.setCond(new FalseCondition());
      break;
    case LPAREN:
      jj_consume_token(LPAREN);
      expression(ctr);
      jj_consume_token(RPAREN);
      break;
    default:
      jj_la1[4] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  final public void negsymbol(NodeContainer ctr) throws ParseException {
        NegCond nc;
        NodeContainer chld;
    jj_consume_token(NEG);
                        nc = new NegCond();
                        chld = new SingleSetter(nc);
                        ctr.setCond(nc);
    symbol(chld);
  }

  /** Generated Token Manager. */
  public ConditionParserTokenManager token_source;
  SimpleCharStream jj_input_stream;
  /** Current token. */
  public Token token;
  /** Next token. */
  public Token jj_nt;
  private int jj_ntk;
  private int jj_gen;
  final private int[] jj_la1 = new int[5];
  static private int[] jj_la1_0;
  static {
      jj_la1_init_0();
   }
   private static void jj_la1_init_0() {
      jj_la1_0 = new int[] {0x520,0xc0,0xc0,0x520,0x500,};
   }

  /** Constructor with InputStream. */
  public ConditionParser(java.io.InputStream stream) {
     this(stream, null);
  }
  /** Constructor with InputStream and supplied encoding */
  public ConditionParser(java.io.InputStream stream, String encoding) {
    try { jj_input_stream = new SimpleCharStream(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source = new ConditionParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 5; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  public void ReInit(java.io.InputStream stream) {
     ReInit(stream, null);
  }
  /** Reinitialise. */
  public void ReInit(java.io.InputStream stream, String encoding) {
    try { jj_input_stream.ReInit(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 5; i++) jj_la1[i] = -1;
  }

  /** Constructor. */
  public ConditionParser(java.io.Reader stream) {
    jj_input_stream = new SimpleCharStream(stream, 1, 1);
    token_source = new ConditionParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 5; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 5; i++) jj_la1[i] = -1;
  }

  /** Constructor with generated Token Manager. */
  public ConditionParser(ConditionParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 5; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  public void ReInit(ConditionParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 5; i++) jj_la1[i] = -1;
  }

  private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }


/** Get the next Token. */
  final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

/** Get the specific Token. */
  final public Token getToken(int index) {
    Token t = token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  private java.util.List jj_expentries = new java.util.ArrayList();
  private int[] jj_expentry;
  private int jj_kind = -1;

  /** Generate ParseException. */
  public ParseException generateParseException() {
    jj_expentries.clear();
    boolean[] la1tokens = new boolean[13];
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 5; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 13; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.add(jj_expentry);
      }
    }
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = (int[])jj_expentries.get(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  /** Enable tracing. */
  final public void enable_tracing() {
  }

  /** Disable tracing. */
  final public void disable_tracing() {
  }

}
