/**(c)

  Copyright (C) 2006, 2007, 2008, 2009 Michael Stilkerich, Johannes Bauer, Christian Wawersich 

  This file is part of the JOSEK operating system.

	JOSEK is free software: you can redistribute it and/or modify it under the
  terms of the Lesser GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your option)
  any later version.

	JOSEK is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
  more details. You should have received a copy of the GNU Lesser General
  Public License along with JOSEK. If not, see <http://www.gnu.org/licenses/>.

  Please contact keso@cs.fau.de for more info.

  (c)**/
package josek.idmap;

public abstract class IDMapObject implements Comparable<IDMapObject> {
	// negative IDs are invalid
	protected final int externalID;
	protected int internalID;
	protected final String name;

	protected IDMapObject(int eid, String name) {
		this.name = name;
		externalID = eid;
		internalID = -1;
	}

	public void setInternalID(int iid) {
		if(internalID >= 0) {
			throw new RuntimeException("Internal ID for " + name + " already set to " +internalID);
		}
		internalID=iid;
	}

	public int getExternalID() { return externalID; }

	public int getInternalID() {
		if(internalID >= 0) {
			return internalID;
		}
		throw new RuntimeException("Internal ID for " + name + " not set yet");
	}

	public String getName() { return name; }

	public int compareTo(IDMapObject imo) {
		if(externalID != imo.externalID)
			return externalID-imo.externalID;

		return name.compareTo(imo.name);
	}

	public String toString() {
		StringBuffer s = new StringBuffer(20);
		s.append(name);
		s.append("(");
		s.append(internalID);
		s.append(")");
		return s.toString();
	}
}

